import 'dart:async';

import 'package:rxdart/rxdart.dart';

import 'base_exception.dart';

/// A class that represents the concepts behind a proper BLoC.
/// Only the fact that said BLoC should have a [Stream] that forwards
/// exceptions to be handled in the user interface has been abstracted.
/// The remaining concepts are explained in the example.
abstract class BaseBloc {
  final BehaviorSubject<BaseException> _exception = BehaviorSubject();

  Observable<BaseException> get exceptionStream => _exception.stream;

  void forwardException(dynamic e) => e is BaseException
      ? (_exception.add(e)) : throw e;

  Future dispose() =>  _exception.close();
}